import { __awaiter } from "tslib";
import { createElement as _createElement } from "@emotion/react";
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "@emotion/react/jsx-runtime";
import { Box, Text, FormControl, Icon, Image, Progress, Flex, IconButton, Modal, ModalOverlay, ModalContent, ModalBody, ModalFooter, Button, } from "@chakra-ui/react";
import { useDropzone } from "react-dropzone";
import React from "react";
import { Upload, Delete, Movie, FileBlank } from "@feedloop/icon";
import { useSource } from "@qorebase/app-runtime/src/hooks/useSource";
import produce from "immer";
import { useFormSubmitHook } from "@qorebase/app-runtime/src/hooks/useFormSubmitHook";
import { RuntimeContext, RuntimeInputContext, useDisableCondition, useTemplate, useClient, useStorageToken, } from "@qorebase/app-component";
import { fallbackImg } from "@qorebase/app-runtime/src/utils/fallbackImage";
import InputComponentFormLabel from "@qorebase/app-runtime/src/components/InputComponentFormLabel";
const MultipleFilePickerComponent = (props) => {
    var _a, _b, _c;
    const title = useTemplate(props.properties.label);
    const placeholder = useTemplate(props.properties.placeholder);
    const fileSize = 4000000;
    const maxFileSizeText = "Maximum File Size: " + `${fileSize / 1000000}` + "MB";
    const [state, setState] = React.useState({ files: {}, loaded: 0, total: 0 });
    const [filesToDelete, setFilesToDelete] = React.useState([]);
    const [errorType, setErrorType] = React.useState("");
    const client = useClient();
    const source = useSource();
    const tables = RuntimeContext.useSelectState((state) => state.schema.tables);
    const views = RuntimeContext.useSelectState((state) => state.schema.views);
    const tableName = React.useMemo(() => {
        var _a, _b;
        return ((_a = tables.find((t) => t.name === source.target)) === null || _a === void 0 ? void 0 : _a.name) ||
            ((_b = views.find((v) => v.name === source.target)) === null || _b === void 0 ? void 0 : _b.query.table);
    }, [tables, views, source.target]);
    const column = props.properties.data.inputID;
    const tableData = React.useMemo(() => {
        const tableIndex = tables === null || tables === void 0 ? void 0 : tables.findIndex((i) => i.name === tableName);
        return tables[tableIndex] ? tables[tableIndex].columns : [];
    }, [tables, tableName]);
    const extension = React.useMemo(() => {
        var _a, _b;
        const columnIndex = tableData === null || tableData === void 0 ? void 0 : tableData.findIndex((i) => i.name === column);
        return ((_a = tableData[columnIndex]) === null || _a === void 0 ? void 0 : _a.type) === "multiple_files" ||
            ((_b = tableData[columnIndex]) === null || _b === void 0 ? void 0 : _b.type) === "file"
            ? tableData[columnIndex].definition.extensions
            : "";
    }, [tableData, column]);
    useFormSubmitHook((row) => __awaiter(void 0, void 0, void 0, function* () {
        if (Object.keys(state.files).length > 0) {
            const uploadToken = yield client.project.axios({
                baseURL: client.project.config.endpoint,
                url: `/v1/files/token/table/${tableName}/id/${row.id}/column/${column}?access=write`,
                method: "GET",
            });
            const formData = new FormData();
            for (const file of Object.values(state.files)) {
                formData.append(file.file.name, file.file);
            }
            yield client.project.axios.post(`/v1/multiple_files/upload?token=${uploadToken.data.token}`, formData, {
                baseURL: client.project.config.endpoint,
                headers: {
                    "Content-Type": "multipart/form-data",
                },
                onUploadProgress: (event) => {
                    setState((state) => produce(state, (draft) => {
                        draft.loaded = event.loaded;
                        draft.total = event.total;
                    }));
                },
            });
        }
        if (filesToDelete.length > 0) {
            yield Promise.all(filesToDelete.map((file) => __awaiter(void 0, void 0, void 0, function* () {
                const deleteToken = yield client.project.axios({
                    baseURL: client.project.config.endpoint,
                    url: `/v1/files/token/table/${tableName}/id/${row.id}/column/${column}?access=delete`,
                    method: "GET",
                });
                return client.project.axios.delete(`/v1/files/${file.filename}?token=${deleteToken.data.token}`, {
                    baseURL: client.project.config.endpoint,
                });
            })));
        }
    }), [tableName, column, state.files, filesToDelete]);
    const storageToken = useStorageToken();
    const getURL = (file) => {
        return storageToken && "url" in file
            ? `${file.url}?token=${storageToken}`
            : `${client.project.config.endpoint}/v1/files/public/${tableName}/${props.data.page.row.id}/${column}/${file.filename}`;
    };
    const handleFileReader = (file) => {
        const reader = new FileReader();
        reader.onload = (e) => {
            Promise.resolve(e.target.result).then((value) => {
                setState((prev) => (Object.assign({}, prev)));
                file["data"] = value;
            });
        };
        reader.readAsDataURL(file);
    };
    const onDrop = React.useCallback((files, rejectedFiles) => __awaiter(void 0, void 0, void 0, function* () {
        if (rejectedFiles.length > 0) {
            if (rejectedFiles[0].errors[0].code === "file-invalid-type") {
                setErrorType(rejectedFiles[0].errors[0].message);
            }
        }
        files.forEach(function (x) {
            handleFileReader(x);
        });
        setState((state) => produce(state, (draft) => {
            for (const file of files) {
                draft.files[file.name] = { file, uploaded: false };
            }
        }));
    }), [source, column]);
    const { getRootProps, getInputProps } = useDropzone({
        onDrop,
        multiple: true,
        accept: extension,
    });
    const paddingSpace = {
        paddingTop: 10,
        maxWidth: "100%",
    };
    const uploadedFiles = React.useMemo(() => (props.data.page.row[props.properties.data.inputID] ||
        []).filter((file) => !filesToDelete.find((fileToDelete) => fileToDelete.filename === file.filename)), [props.data.page.row[props.properties.data.inputID], filesToDelete]);
    const filesToUpload = React.useMemo(() => Object.values(state.files), [state.files]);
    const inputActions = RuntimeInputContext.useActions();
    const [validationMessage, setValidationMessage] = React.useState("");
    const isRequired = (_a = props.properties.data.validation) === null || _a === void 0 ? void 0 : _a.required;
    React.useEffect(() => {
        const totalFiles = uploadedFiles.length + filesToUpload.length;
        if (isRequired && totalFiles === 0) {
            const message = `${title} require at least one file.`;
            inputActions.setValidation({
                [column]: { error: message },
            });
            setValidationMessage(message);
        }
        else {
            inputActions.setValidation({
                [column]: undefined,
            });
            setValidationMessage("");
        }
    }, [uploadedFiles, filesToUpload, isRequired]);
    const arrSize = Object.keys(state.files).length;
    const currFileSize = (_b = Object.values(state.files)[arrSize - 1]) === null || _b === void 0 ? void 0 : _b.file.size;
    // @ts-ignore
    const { isDisabled } = useDisableCondition(props.instance);
    return (_jsxs(Box, Object.assign({ display: props.properties.hidden ? "none" : undefined }, { children: [_jsxs(FormControl, { children: [_jsx(InputComponentFormLabel, { title: title, name: "multiple-file-picker", instanceId: props.instance.id, fontWeight: "normal", isRequired: (_c = props.properties.data.validation) === null || _c === void 0 ? void 0 : _c.required, isEmpty: title.length === 0 }), !!uploadedFiles.length && (_jsxs(Box, Object.assign({ mt: 2, border: "1px", borderRadius: "md", borderColor: "surface.900", paddingY: "5", paddingX: "3" }, { children: [_jsx(Box, Object.assign({ mb: 2 }, { children: _jsx(Text, Object.assign({ fontWeight: "semibold" }, { children: "Uploaded files" })) })), uploadedFiles.map((file) => {
                                var _a;
                                return (_jsxs(Flex, Object.assign({ alignItems: "center" }, { children: [_jsx(Box, Object.assign({ display: "flex", width: "48px", height: "48px", bgColor: `${props.app.theme.options.primaryColor}23`, marginRight: 5, justifyContent: "center", alignItems: "center", borderRadius: "4px", marginTop: 2 }, { children: file.mimetype.startsWith("image") ? (_jsx(Image, { alt: "New image", src: (_a = getURL(file)) !== null && _a !== void 0 ? _a : fallbackImg, width: "48px", height: "48px", borderRadius: "4px" })) : file.mimetype.startsWith("video") ? (_jsx(Icon, { as: Movie, boxSize: 7, color: `${props.app.theme.options.primaryColor}` })) : file.mimetype === "application/pdf" ? (_jsx(Icon, { as: FileBlank, boxSize: 7, color: `${props.app.theme.options.primaryColor}` })) : (_jsx(Icon, { as: Upload, boxSize: 7, color: `${props.app.theme.options.primaryColor}` })) })), _jsxs(Box, Object.assign({ flex: 1 }, { children: [_jsx(Text, Object.assign({ noOfLines: 1 }, { children: file.filename })), _jsxs(Text, Object.assign({ fontSize: "sm", textColor: "text.600" }, { children: [Math.ceil(file.size / 1024), " KB"] }))] })), _jsx(IconButton, { "aria-label": "delete file", icon: _jsx(Icon, { as: Delete }), variant: "link", color: "#171717", _hover: { color: "#E03138", bg: "#FCEAEB" }, width: "48px", height: "48px", onClick: () => setFilesToDelete((files) => files.concat(file)) })] }), file.filename));
                            })] }))), _jsxs(Modal, Object.assign({ isOpen: currFileSize > fileSize, isCentered: true, size: "xs", onClose: () => { } }, { children: [_jsx(ModalOverlay, {}), _jsxs(ModalContent, Object.assign({ paddingY: "5" }, { children: [_jsxs(ModalBody, { children: [_jsx(Text, Object.assign({ textAlign: "center", fontSize: "xl", fontWeight: "bold" }, { children: "Sorry, file size is too big" })), _jsx(Text, Object.assign({ textAlign: "center", fontSize: "sm", color: "blackAlpha.500" }, { children: maxFileSizeText }))] }), _jsx(ModalFooter, { children: _jsx(Button, Object.assign({ onClick: () => setState((prev) => (Object.assign(Object.assign({}, prev), { files: {} }))), w: "full" }, { children: "Ok" })) })] }))] })), _jsxs(Modal, Object.assign({ isOpen: !!errorType, onClose: () => { }, isCentered: true, size: "xs" }, { children: [_jsx(ModalOverlay, {}), _jsxs(ModalContent, Object.assign({ paddingY: "5" }, { children: [_jsxs(ModalBody, { children: [_jsx(Text, Object.assign({ textAlign: "center", fontSize: "xl", fontWeight: "bold" }, { children: "Sorry, file type invalid" })), _jsx(Text, Object.assign({ textAlign: "center", fontSize: "sm", color: "blackAlpha.500" }, { children: errorType }))] }), _jsx(ModalFooter, { children: _jsx(Button, Object.assign({ onClick: () => setErrorType(""), w: "full" }, { children: "Ok" })) })] }))] })), !!filesToUpload.length && (_jsxs(Box, Object.assign({ mt: 2, border: "1px", borderRadius: "md", borderColor: "surface.900", paddingY: "5", paddingX: "3" }, { children: [_jsx(Box, Object.assign({ mb: 2 }, { children: _jsx(Text, Object.assign({ fontWeight: "semibold" }, { children: "Files to upload" })) })), filesToUpload.map((file) => {
                                var _a;
                                return (_jsxs(Flex, Object.assign({ alignItems: "center" }, { children: [_jsx(Box, Object.assign({ display: "flex", width: "48px", height: "48px", bgColor: `${props.app.theme.options.primaryColor}23`, marginRight: 5, justifyContent: "center", alignItems: "center", borderRadius: "4px", marginTop: 2 }, { children: file.file.type.startsWith("image") ? (_jsx(Image, { alt: "New image", src: (_a = file.file["data"]) !== null && _a !== void 0 ? _a : fallbackImg, width: "48px", height: "48px", borderRadius: "4px" })) : file.file.type.startsWith("video") ? (_jsx(Icon, { as: Movie, boxSize: 7, color: `${props.app.theme.options.primaryColor}` })) : file.file.type === "application/pdf" ? (_jsx(Icon, { as: FileBlank, boxSize: 7, color: `${props.app.theme.options.primaryColor}` })) : (_jsx(Icon, { as: Upload, boxSize: 7, color: `${props.app.theme.options.primaryColor}` })) })), _jsxs(Box, Object.assign({ flex: 1 }, { children: [_jsx(Text, Object.assign({ noOfLines: 1 }, { children: file.file.name })), _jsxs(Text, Object.assign({ fontSize: "sm", textColor: "text.600" }, { children: [Math.ceil(file.file.size / 1024), " KB"] }))] })), _jsx(IconButton, { "aria-label": "delete file", icon: _jsx(Icon, { as: Delete }), variant: "link", color: "#171717", _hover: { color: "#E03138", bg: "#FCEAEB" }, width: "48px", height: "48px", onClick: () => {
                                                setState((state) => (Object.assign(Object.assign({}, state), { files: Object.fromEntries(Object.entries(state.files).filter(([key]) => key !== file.file.name)) })));
                                            } })] }), file.file.name));
                            }), !!state.total && (_jsxs(_Fragment, { children: [_jsx(Progress, { value: Math.round((state.loaded / state.total) * 100) }), _jsxs(Text, Object.assign({ fontSize: "sm", textColor: "blackAlpha.600" }, { children: [Math.ceil(state.loaded / 1024), "/", Math.ceil(state.total / 1024), " KB"] }))] }))] }))), _jsxs(Box, Object.assign({}, getRootProps({ className: "dropzone" }), { display: "flex", flexDirection: "column", alignItems: "center", mt: 2, cursor: "pointer", border: "1px", borderRadius: "md", borderColor: "rgb(226, 232, 240)", paddingY: "10", bgColor: "surface.900", style: {
                            border: "1px solid rgb(226, 232, 240)",
                            pointerEvents: isDisabled ? "none" : "auto",
                            background: isDisabled && "#E7E7E7",
                            color: isDisabled && "#A2A2A2",
                        } }, { children: [_createElement("input", Object.assign({}, getInputProps(), { key: props.properties.data.inputID, name: props.properties.data.inputID, capture: props.properties.fileType === "camera"
                                    ? "user" || "environment"
                                    : undefined, "data-test": "file-picker-input" })), _jsx(Icon, { as: Upload, boxSize: 10, color: isDisabled ? "#A2A2A2" : "blue.500", "data-test": "file-picker-input-icon" }), _jsx(Text, Object.assign({ style: paddingSpace, noOfLines: 1, align: "center", w: { base: "300px", md: "500px" }, color: isDisabled && "#A2A2A2" }, { children: placeholder })), _jsx(Text, Object.assign({ fontSize: "sm", color: "text.600" }, { children: "Support for a single and bulk upload." })), _jsx(Text, Object.assign({ fontSize: "sm", color: "text.600" }, { children: maxFileSizeText }))] }))] }), !!validationMessage && (_jsx(Text, Object.assign({ fontSize: "sm", color: "red", mt: "2" }, { children: validationMessage })))] })));
};
export default MultipleFilePickerComponent;
//# sourceMappingURL=MultipleFilePickerComponent.js.map