import { Box, Text } from "@chakra-ui/react";
import { FileError } from "react-dropzone";
import { defineComponent } from "@qorebase/app-component";
import { TableSchema, ViewSchema } from "@qorebase/app-runtime/src/interfaces";
import React from "react";
import { useHumanize } from "@qorebase/app-runtime/src/utils/humanizeString";
import { ComponentPropertyEditorContainer } from "@qorebase/app-runtime/src/components/ComponentPropertyEditorContainer";
import MultipleFilePickerComponent from "./MultipleFilePickerComponent";

export interface RejectedFiles {
  file: File;
  errors: FileError[];
}

export type CropProps = {
  unit: string;
  height: number;
  aspect: number;
};

export interface FileUpload {
  file: File;
  uploaded: boolean;
}

const MultipleFilePicker = defineComponent({
  name: "Multiple File Picker",
  type: "none",
  icon: "MultipleFiles",
  group: "input",
  display: "inline",
  defaultProps: {
    label: "Multiple File Picker",
    placeholder: "Click or drag file here to upload",
    hidden: false,
    data: {
      inputID: "multiplefilepicker",
      field: { type: "string", options: { format: "text" } },
      validation: {
        required: false,
      },
    },
    fileType: [],
  },
  propDefinition: {
    label: { group: "Design", type: "string", options: { format: "text" } },
    placeholder: {
      group: "Design",
      type: "string",
      options: { format: "text" },
    },
    hidden: {
      group: "Design",
      type: "boolean",
      options: { format: "boolean" },
    },
    data: {
      group: "Design",
      type: "state",
      options: {
        inputID: "multiplefilepicker",
        field: { type: "string", options: { format: "text" } },
      },
    },
    fileType: {
      group: "Design",
      type: "custom",
      options: {
        style: "full-width",
      },
    },
  },
  Editor: {
    fileType: (props) => {
      const column = React.useMemo(
        () => props.values.data.inputID,
        [props.values.data.inputID]
      );
      const columnList = React.useMemo(() => {
        if (
          !!props.view &&
          !!props.view.columns &&
          Array.isArray(props.view.columns)
        ) {
          const view = props.view as TableSchema | ViewSchema;
          return view.columns.filter(
            (c) =>
              c.name === column &&
              (c.type === "file" || c.type === "multiple_files")
          );
        }
        return [];
      }, [props.view, column]);

      const extension = React.useMemo(
        () => (columnList.length ? columnList[0].definition.extensions : []),
        [columnList]
      );
      const title = useHumanize(props.propDef.label ?? props.propName);
      const desc =
        "File type cannot be changed, the data here is based on file extensions column setting type file / multiple files in Qorebase table";

      if (extension.length === 0) return null;

      return (
        <ComponentPropertyEditorContainer title={title} description={desc}>
          <Box py={2}>
            <Text
              fontSize="sm"
              padding={4}
              color="gray.600"
              borderRadius="md"
              backgroundColor="#EDF2F7"
            >
              {extension}
            </Text>
          </Box>
        </ComponentPropertyEditorContainer>
      );
    },
  },
  Component: (props) => <MultipleFilePickerComponent {...props} />,
});

export type MultipleFilePickerComponentType =
  typeof MultipleFilePicker["Component"];

export default MultipleFilePicker;
